@@ -253,9 +253,9 @@ def generate_exec_filesystem(protocol_triad, cache_dir, job_data, runtime_root_p
 def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
         cpus_per_node, requested_nodes, batch_job_code, fs_locations, 
         data_transfer_protocol, singinfo=(None, None, None), task_cd=None,
-        email_address="", email_type="ALL", tasks_per_core=1,
-        nodescratch_folder="", nodescratch_mem="", walltime="24:00:00",
-        outer_statements="", exclusive=False):
+        email_address="", email_type="ALL", tasks_per_core=1, min_stack_per_core=10,
+        constraint='', nodescratch_folder="", nodescratch_mem="", walltime="24:00:00",
+        outer_statements="", exclusive=False, mem='', mempercpu=''):
 
     protocol, remote_machine, hpc_shared_folder = protocol_triad
     devnull = open('/dev/null', 'w')
@@ -283,6 +283,10 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
     if exclusive:
         turnon_exclusiveness = ""
 
+    turnon_constraint = "#"
+    if constraint:
+        turnon_constraint = ""
+
     if len(walltime) < 7 or walltime[-3] != ":" or walltime[-6] != ":":
         print("ERROR (manager): walltime is not well formatted: XX:XX:XX")
         print("                 "+walltime)
@@ -309,13 +313,23 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
         requested_nodes, 
         len(taskid_list)
     )
+    
+    if mem:
+        turnon_mem = ""
+    else:
+        turnon_mem = "#"
+
+    if mempercpu:
+        turnon_mempercpu = ""
+    else:
+        turnon_mempercpu = "#"
 
     # Reassess the number of requested nodes for avoiding asking
     #  for too many nodes. For requiring another node, all cpus of all
     #  nodes must have at least 10 tasks assigned to them
     #  WARNING: this does not keep into account the length of a task!
-    if protocol != 'local' and min(tasks_per_node_list) < 10*cpus_per_node:
-        grand = 10*cpus_per_node / max(tasks_per_node_list)
+    if protocol != 'local' and min(tasks_per_node_list) < min_stack_per_core*cpus_per_node:
+        grand = min_stack_per_core*cpus_per_node / max(tasks_per_node_list)
         rqn = (
             int(requested_nodes / grand)
             + max(tasks_per_node_list) - min(tasks_per_node_list)
@@ -360,8 +374,10 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
                 .replace('<turnonmailtype>', turnon_mailtype) \
                 .replace('<mailtype>', email_type) \
                 .replace('<turnonemailaddress>', turnon_email_address) \
+                .replace('<turnonconstraint>', turnon_constraint) \
                 .replace('<turnonexclusiveness>', turnon_exclusiveness) \
                 .replace('<emailaddress>', email_address) \
+                .replace('<constraint>', constraint) \
                 .replace('<outpath>', outpath) \
                 .replace('<errpath>', errpath) \
                 .replace('<taskfile>', task_filename) \
@@ -371,7 +387,11 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
                 .replace('<extra_outer_statements>', outer_statements) \
                 .replace('<singularity_module_load>', singmodload) \
                 .replace('<singularity_command>', singularitycmd) \
-                .replace('<inner_manager>', "inner_manager_{0}{1}.slurm".format(batch_job_code, str(jobid).zfill(3)))
+                .replace('<inner_manager>', "inner_manager_{0}{1}.slurm".format(batch_job_code, str(jobid).zfill(3))) \
+                .replace('<turnonmem>', turnon_mem) \
+                .replace('<mem>', mem) \
+                .replace('<turnonmempercpu>', turnon_mempercpu) \
+                .replace('<mempercpu>', mempercpu)
 
             # Write manager file and give it exe privilege
             manager_filename = (
@@ -396,6 +416,26 @@ def create_manager_scripts(protocol_triad, cache_dir, task_folders, partition,
         # Job manager identifier and associated tasks
         tasks_per_job.append((jobid, taskid_list[ik:ik+tasks_per_node]))
 
+    manager_cmd, macname = [], {}
+    manager_cmd.append("echo MANAGERS")
+    for job_id, task_list in tasks_per_job:
+        macname[job_id] = "{0}/.manager_activity_check_{1}{2}".format(
+            fs_locations["runtime_exec"],
+            batch_job_code,
+            str(job_id).zfill(3)
+        )
+        manager_cmd.append(" ".join(["date", "-r", macname[job_id], "+%H:%M", "2>&1"]))
+    manager_cmd.append("echo MACNAMES")
+    for job_id, task_list in tasks_per_job:
+        manager_cmd.append(" ".join(["cat", macname[job_id], "2>&1"]))
+    manager_cmd.append("echo STATUS")
+    manager_cmd.append('squeue --format="%.18i %.9P %.100j %.8u %.2t %.10M %.6D %R"')
+    manager_cmd.append("echo TIME")
+    manager_cmd.append("echo $(date +%H:%M)")
+    with open(fs_locations["build_exec"] + 'periodic_check.sh', 'w') as cmf:
+        cmf.write("\n".join(manager_cmd))
+    subprocess.call(["chmod", "777", fs_locations["build_exec"] + 'periodic_check.sh'])
+
     if protocol == 'remote':
         cpall_cmd = ["bash", data_transfer_protocol, fs_locations["build_root"], "{0}:{1}".format(remote_machine, fs_locations["runtime_root"])]
         if DEBUG:
@@ -432,7 +472,7 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
                     .format(batch_job_code, str(job_id).zfill(3))
             ]
             if DEBUG:
-                print("COMMAND", chk_cmd)
+                print("COMMAND", " ".join(chk_cmd))
                 p = subprocess.Popen(chk_cmd)
             else:
                 p = subprocess.Popen(chk_cmd,
@@ -459,48 +499,120 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
     chrono_on = {job_id : False for job_id, task_list in tasks_per_job}
     time_start = {job_id : 0 for job_id, task_list in tasks_per_job}
     time_end = {job_id : 0 for job_id, task_list in tasks_per_job}
+    machine_names = {job_id : 'NoHostname' for job_id, task_list in tasks_per_job}
+    job_times = {job_id : 0 for job_id, task_list in tasks_per_job}
+    hasrun = {job_id : False for job_id, task_list in tasks_per_job}
+    sc, wc = 0, 0
     while not is_over:
+        status = {job_id : '' for job_id, task_list in tasks_per_job}
         time.sleep(waiting_time)
         is_over = True
-        for job_id, task_list in tasks_per_job:
 
+        # Start/stop chronometer
+        for job_id, task_list in tasks_per_job:
             if chrono_on[job_id] and not time_start[job_id]:
                 time_start[job_id] = time.time()
             if not chrono_on[job_id] and time_start[job_id]:
                 time_end[job_id] = time.time()
 
-            macname = "{0}/.manager_activity_check_{1}{2}".format(
-                fs_locations["runtime_exec"], 
-                batch_job_code, 
-                str(job_id).zfill(3)
-            )
-            macname_sf = "{0}/.manager_activity_check_{1}{2}".format(
-                fs_locations["build_exec"], 
-                batch_job_code, 
-                str(job_id).zfill(3)
-            )
+        if protocol != 'local':
+            manager_cmd = ["ssh", remote_machine, "bash", fs_locations["runtime_exec"] + 'periodic_check.sh']
+        else:
+            manager_cmd = ["bash", fs_locations["build_exec"] + 'periodic_check.sh']
 
-            if protocol == 'remote':
-                manager_cmd = ["ssh", remote_machine, "date", "-r", macname, "+%H:%M"]
-                chk_time_cmd = "ssh " + remote_machine + " echo $(date +%H:%M)"
-            elif protocol == 'remote-sharedfs':
-                manager_cmd = ["date", "-r", macname_sf, "+%H:%M"]
-                chk_time_cmd = "echo $(date +%H:%M)"
-            elif protocol == 'local':
-                manager_cmd = ["date", "-r", macname, "+%H:%M"]
-                chk_time_cmd = "echo $(date +%H:%M)"
+        if DEBUG:
+            print("COMMAND", manager_cmd)
+            periodic_check_txt = subprocess.Popen(
+                manager_cmd,
+                stdout=subprocess.PIPE
+            ).stdout.read().decode('utf-8')
+        else:
+            periodic_check_txt = subprocess.Popen(
+                manager_cmd, 
+                stderr=devnull, 
+                stdout=subprocess.PIPE
+            ).stdout.read().decode('utf-8')
+
+        mng_lines, mac_lines, status_lines, time_line = False, False, False, False
+        mng_times, mac_names = [], []
+        for line in periodic_check_txt.split('\n'):
+            if not line.strip():
+                continue
+            if line == "MANAGERS":
+                mng_lines = True
+                continue
+            elif line == "MACNAMES":
+                mng_lines = False
+                mac_lines = True
+                continue
+            elif line == "STATUS":
+                mac_lines = False
+                status_lines = True
+                continue
+            elif line == "TIME":
+                status_lines = False
+                time_line = True
+                continue
+            if mng_lines:
+                mng_times.append(line.strip())
+            elif mac_lines:
+                mac_names.append(line.strip()) 
+            elif status_lines:
+#                print(line)
+                if line.split()[2] in [batch_job_code + str(x[0]).zfill(3) for x in tasks_per_job]:
+#                    print("YE", line)
+                    status[int(line.split()[2][len(batch_job_code):])] = line.split()[4]
+                    hasrun[int(line.split()[2][len(batch_job_code):])] = True
+            elif time_line:
+                real_time = line.strip()
+                real_time = int(real_time.split(":")[0])*60 + int(real_time.split(":")[1])
+
+
+        for it, t in enumerate(tasks_per_job):
+            job_id, task_list = t
+
+            if len(mng_times[it].split()) == 1:
+                job_times[job_id] = int(mng_times[it].split(":")[0])*60 + int(mng_times[it].split(":")[1])
+#            else:
+#                print("WARNING: missing manager", it, job_id, mng_times[it])
+
+            if len(mac_names[it].split()) == 1:
+                machine_names[job_id] = mac_names[it]
+#            else:
+#                print("WARNING: missing machine name", it, job_id, mac_names[it])
+
+            if status[job_id]:
+                print(job_id, "Status:", status[job_id])
+                is_over = False
+            elif not hasrun[job_id]:
+                print(job_id, "Waiting to be queued ({0}/3)".format(wc+1))
+                wc += 1
+                if wc > 3:
+                    print(job_id, "Abort")
+                else:
+                    is_over = False
+            else:
+                print(job_id, "End")
+                time_end[job_id] = job_times[job_id]
+
+        continue
+
+        if machine_names[job_id] == "NoHostname":
             if DEBUG:
-                print("COMMAND", manager_cmd)
-                touch_time_txt = subprocess.Popen(
-                    manager_cmd,
+                print("COMMAND", machinename_cmd)
+                mname_txt = subprocess.Popen(
+                    machinename_cmd,
                     stdout=subprocess.PIPE
                 ).stdout.read().decode('utf-8')
             else:
-                touch_time_txt = subprocess.Popen(
-                    manager_cmd, 
-                    stderr=devnull, 
-                    stdout=subprocess.PIPE
+                mname_txt = subprocess.Popen(
+                    machinename_cmd,
+                    stdout=subprocess.PIPE,
+                    stderr=devnull
                 ).stdout.read().decode('utf-8')
+            if mname_txt.strip():
+                machine_names[job_id] = mname_txt.split()[1]
+
             if touch_time_txt.strip():
                 if DEBUG:
                     print("COMMAND", chk_time_cmd)
@@ -523,7 +635,7 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
                 if is_active:
                     is_over = False
                     chrono_on[job_id] = True
-                    print("Job", job_id, "running")
+                    print("Job", job_id, "running", machine_names[job_id])
                 else:
                     chrono_on[job_id] = False
                     print("Job", job_id, "ended")
@@ -565,20 +677,21 @@ def remote_job_control(protocol_triad, batch_job_code, fs_locations, tasks_per_j
                 else:
                     print("Job", job_id, "error/aborted")
 
-    log_txt = "\n\n#JOBS LOG\n"
+    log_txt = "\n\n#LOCUSTS JOBS LOG\n"
     time_now = time.time()
     tot_runtime, totsq = 0, 0
     for job_id, task_list in tasks_per_job:
-        if not time_end[job_id]: 
-            time_end[job_id] = time_now
-        runtime = time_end[job_id] - time_start[job_id]
+        if not time_end[job_id] - time_start[job_id] == 0:
+            runtime = waiting_time/2
+        else: 
+            runtime = time_end[job_id] - time_start[job_id]
         tot_runtime += runtime
         totsq += runtime**2
-        log_txt += "JobID {0} :\tRuntime {1}\tRuntime (seconds) {2}\tTasks {3}\n".format(job_id, datetime.timedelta(seconds = int(runtime)), int(runtime), len(task_list))
+        log_txt += "LocustsJobID {0} :\tHostname {1}\tRuntime {2}\tRuntime (seconds) {3}\tTasks {4}\n".format(job_id, machine_names[job_id], datetime.timedelta(seconds = int(runtime)), int(runtime), len(task_list))
 
     av_runtime = tot_runtime/len(tasks_per_job)
     stdev_runtime = (totsq/len(tasks_per_job) - av_runtime**2)**0.5
-    log_txt += "Total computing time :\t{0}\tSeconds {1}\n".format(datetime.timedelta(seconds = int(tot_runtime)), int(tot_runtime))
+    log_txt += "\n\nTotal computing time :\t{0}\tSeconds {1}\n".format(datetime.timedelta(seconds = int(tot_runtime)), int(tot_runtime))
     log_txt += "Average computing time per node:\t{0}\tSeconds {1}\n".format(datetime.timedelta(seconds = int(av_runtime)), int(av_runtime))
     log_txt += "Time imbalance (std dev):\t{0}\tSeconds {1}\n".format(datetime.timedelta(seconds = int(stdev_runtime)), int(stdev_runtime))
 
@@ -628,7 +741,7 @@ def gather_results(protocol_triad, cache_dir, job_data, batch_job_code,
 
         # Check whether any of the task results still pending (not executed)
         #  or running (might have been interrupted). If so, adds to the reschedule set
-        sname = "status_{0}{1}".format(batch_job_code, str(job_id).zfill(3))
+        sname = "status_{0}{1}.txt".format(batch_job_code, str(job_id).zfill(3))
         status_filename = fs_locations["runtime_exec"] + sname
         grep_pending_cmd = ["grep", "'pending'", status_filename]
         if remote_machine:
@@ -949,19 +1062,27 @@ def highly_parallel_job_manager(options, exec_filename,
     out_st = options['extra_outer_statements']
     partition = options['partition']
     exclusive = options['exclusive']
+    task_stack = options['min_stack_per_core']
+    constraint = options['constraint']
+    waiting_time = options['waiting_time']
+    memory = options['memory']
+    memory_per_cpu = options['memory_per_cpu']
 
     log_txt = ("#RUN LOG\n\n"
         "Run type (local / shared / remote) :\t{0}\n").format(protocol)
     if protocol != 'local':
         log_txt += ("Remote machine :\t{0}\n"
             "Partition (queue) :\t{1}\n"
-            "Number of reserved nodes :\t{2}\n"
-            "Number of cores per node :\t{3}\n"
-            "Exclusive use of nodes :\t{4}\n"
-            "Wall time :\t{5}\n"
-            "Nodewise scratch folder :\t{6}\n"
-            "Nodewise scratch memory :\t{7}\n"
-        ).format(remote_machine, partition, requested_nodes, cpus_per_node, exclusive, wt, nsf, nsm)   
+            "Constraint (node type) :\t{2}\n"
+            "Number of reserved nodes :\t{3}\n"
+            "(Minimum) number of cores per node :\t{4}\n"
+            "Exclusive use of nodes :\t{5}\n"
+            "Tasks :\t{6}\n"
+            "Minimum number of tasks assigned per core :\t{7}\n"
+            "Wall time :\t{8}\n"
+            "Nodewise scratch folder :\t{9}\n"
+            "Nodewise scratch memory :\t{10}\n"
+        ).format(remote_machine, partition, constraint, requested_nodes, cpus_per_node, exclusive, len(job_data), task_stack, wt, nsf, nsm)   
     else:
         log_txt += "Number of local processors :\t{0}\n".format(cpus_per_node)
     print(log_txt)
@@ -992,11 +1113,15 @@ def highly_parallel_job_manager(options, exec_filename,
             data_transfer_protocol,
             singinfo=singinfo,
             email_address=email,
+            min_stack_per_core=task_stack,
+            constraint=constraint,
             nodescratch_folder=nsf,
             nodescratch_mem=nsm,
             walltime=wt,
             outer_statements=out_st,
-            exclusive=exclusive
+            exclusive=exclusive,
+            mem=memory,
+            mempercpu=memory_per_cpu
         )
         log_txt += tmp_log_txt
 
@@ -1019,7 +1144,8 @@ def highly_parallel_job_manager(options, exec_filename,
         # Create extrenal master script that checks out from time to time (each 5 mins or so)
         # This step is over only when no process is active anymore
         lenlist = [len(x[1]) for x in tasks_per_job]
-        waiting_time = min(600, 10*(1+len(job_data)//min(lenlist)))
+        if waiting_time == -1:
+            waiting_time = min(600, 10*(1+len(job_data)//min(lenlist)))
         log_txt += remote_job_control(
             protocol_triad,
             batch_job_code, 
